// Datei: LCD.C
// 30MHz-PLL_Rechteckgenerator mit PIC16C84
//
// Ansteuerung der LCD-Anzeige
// im 4 Bit Daten Mode
//
// Copyright 1998 by Holger Klabunde
// 30.04.2001
// Compiler CC5x

#include "prozess.h"      /* Der verwendete Prozessor */
#include "pllser84.h" //Pindefinitionen
#include "protos.h"

//*******************************************
// Schreibt ein Datenbyte in die LCD-Anzeige
// Schreibt das Byte in zwei 4 Bit Nibbles
//*******************************************
void LCDWriteByte(unsigned char data)
{
 LCD_CS=0;

 j=PORTB; //den letzten Zustand von PORTB lesen

 result=(data>>4)&0x0F; //High Nibble des Bytes
 j=j&0xF0;		//Hier D4-D7 ersma lschen zum
 j|=result;		//reinodern vom High Nibble
 PORTB=j;		//Auf den Port legen und
 NOP;

 LCD_CS=1;	//High Nibble schreiben
 NOP;
 LCD_CS=0;	

 result=data&0x0F;	//Low Nibble des Bytes
 j=j&0xF0;		//Hier D4-D7 ersma lschen zum
 j|=result;		//reinodern vom Low Nibble
 PORTB=j;		//Auf den Port legen und
 NOP;

 LCD_CS=1;	//Low Nibble schreiben
 NOP;
 LCD_CS=0;
}

//**************************************************
//Schreibt einen Text in die LCD-Anzeige
//**************************************************
void LCDWrite(const char *__putstr)
{
  char __ps;

  __ps = *__putstr;

  while(__ps >0)
  {
   __putstr++;
   if (__ps== 0) break;
      LCDWriteByte(__ps);
   __ps = *__putstr;
  }
}


//****************************************************
// Springt auf Zeile 'x' Position 'y' der LCD-Anzeige 
// Gltig fr 2 Zeilen
//****************************************************
void LCDPos(unsigned char x,unsigned char y)
{
 LCD_RS=0; //Umschalten auf Kontrollregister

// 0x80 wegen Display Adress Set.
// Erste Position ist bei mir 1, nicht 0.
// Offset's
// Zeile 1 -> 0x00+0x80-0x01=0x7F
// Zeile 2 -> 0x40+0x80-0x01=0xBF 

 switch(x)
  {
   case 1: LCDWriteByte(0x7F+y); break;
   case 2: LCDWriteByte(0xBF+y); break;
   default : break;
  }

 LCD_RS=1; //Zurckschalten auf Daten
 Delay1ms(1);
}

//*****************************************
// Lscht die LCD Anzeige
// Cursor steht dann auf Position 1,Zeile 1
// Gilt auch fr mehrzeilige Anzeigen
//*****************************************
void LCDCls()
{
 LCD_RS=0; //Controlregister
 LCDWriteByte(0x01); //CLS
 LCD_RS=1; //Zurck auf Datenregister
 Delay1ms(5);
}


//*********************************************
//Stellt die Funktionsweise der LCD Anzeige ein
//*********************************************
void InitLCD()
{
 LCD_RS=0;		//Control Register
 NOP;
 LCD_CS=0;        //LCD-Anzeige anwhlen
 NOP;

//Der erste Schreibzugriff nach dem Reset ist ein 8 Bit
//Zugriff!!

 Delay1ms(200); //Powerup abwarten

//Um die 4Bit Operation einzuschalten
 j=PORTB; 		//den letzten Zustand von PORTB lesen
 j=j&0xF0;		//D4-D7 ersma lschen
 j|=0x02;		//4 Bit Operation
 PORTB=j;		//Auf den Port legen und
 NOP;
 LCD_CS=1;	//High Nibble schreiben
	

//Nach einfgen von Wait() = 5ms
//Kommt die LCD Anzeige immer nach dem Einschalten
//Ohne Wait's zeigte sie oft gar nix
 LCDWriteByte(0x20);    //4 Bit Operation,1 Line,5x7 Font
 Delay1ms(5);
 LCDWriteByte(0x14); 	//Cursor Move,Right Shift
 Delay1ms(5);
 LCDWriteByte(0x0E); 	//Display on, Cursor on, Cursorblink off
 Delay1ms(5);
 LCDWriteByte(0x06); 	//Increment, Display Freeze
 Delay1ms(5);
 LCDWriteByte(0x02); 	//Cursor Home
 Delay1ms(5);
 LCD_RS=1;		//LCD-Datenregister
}

