//Datei: PLLSER84.C
// Hauptprogramm
// 30MHz-PLL_Rechteckgenerator mit PIC16C|F84
// 100Hz-30MHz in 100Hz Schritten
//
// Neue Version mit Tastaturabfrage im Interrupt
// und automatischem Rauf-Runterzhlen nach 2s
//
// Copyright 1998 by Holger Klabunde
// 01.05.2001
// Compiler CC5x

// Achtung:	int ist 8 Bit lang
//		char ist 8 Bit lang
//		long ist 16 Bit lang

// Der Takt des PIC betrgt 204.8kHz
// Ein Maschinenzyklus damit 19.5us

// Verwendete LCD-Anzeige LM027 (1x24 Zeichen)
// Bentigt werden nur 1x16 Zeichen !!
// Anzeige 	12345.6 kHz

#include "prozess.h"      /* Der verwendete Prozessor */
#include "pllser84.h" //Pindefinitionen
#include "protos.h"

#include "interupt.c"
#include "delay.c"
#include "lcd.c"
#include "eeprom.c"
#include "setpll.c"
#include "initpic.c"
#include "showdisp.c"
#include "chekdisp.c"

//Globale Flags und Rckgabevariablen

unsigned char stelle; 	//Position auf der LCD-Anzeige
unsigned char key;	//Zum Lesen der Tastatur
unsigned char i,j;	//Alle mglichen Schleifen
unsigned char result;	//Zwischenergebnisse
unsigned char tastbuf;  //Tastaturpuffer fr lesen im INT
unsigned char boing;    //Warten auf INT
unsigned char rep;      //Mehrfachauslsen vermeiden

unsigned char display[7];  //BCD Werte des Generators
unsigned char backup[7];   //Kopie fr display

/* Hauptprogramm */
void main()
{
 InitPIC(); //Interrupts und so weiter
 InitLCD(); //LCD Anzeige einstellen

 LCDCls();  //Anzeige lschen
 LCDWrite("Holgis 30Mhz PLL");
 Delay1ms(255);          //Anzeige etwas verlngern
 Delay1ms(255);          //Anzeige etwas verlngern
 Delay1ms(255);          //Anzeige etwas verlngern
 Delay1ms(255);          //Anzeige etwas verlngern

 LCDCls();  //Anzeige lschen

// Erstmal die gespeicherte Frequenz aus dem EEPROM lesen !!
 for(i=1; i<7; i++)
  {
   result=Read_EEProm(i);
   display[i]=result;
   if(result>9) display[i]=0;
  }

 stelle=Read_EEProm(0);
 if((stelle>6)||(stelle==0)) stelle=5; //1kHz Position

 SetPLL(); //Teiler programmieren

 ShowDisplay();

 boing=0; //Flag "TIMERINT war da", sonst wird in der Schleife
 	  //key mehrfach bearbeitet !
 rep=0;   //Merkt sich den letzen Wert von boing
 	  
 while(1) // Anfang Programm Endlosschleife
  {

   key=0;   //Tastaturinhalt lschen

   if(boing>0) //Taste gedrckt ?
    {
     if(rep!=boing) //War ein neuer INT da ?
      {
       rep=boing;     //nchstes Auslsen erst wieder wenn neuer INT da war

       //Einzelauslsung bzw. langsam hochzhlen
       if(boing==1) key=tastbuf; //Tastaturpuffer lesen, Lowaktiv

       //Ab hier schnelles Tastaturrepeat
       if(boing>9) key=tastbuf;  
      }
    }
   else rep=0; //Taste losgelassen, Tastenwiederholung lschen
   
   if(key==RIGHT)
    {
     if(stelle<7) stelle=stelle+1; // Eins nach rechts
     if(stelle==7) stelle=1; // Ans linke Ende des Displays bei berlauf
     ShowDisplay(); 		//Display updaten
    }

   if(key==LEFT)
    {
     if(stelle>0) stelle=stelle-1; // Eins nach links
     if(stelle==0) stelle=6; // Ans rechte Ende des Displays
     ShowDisplay();          //Display updaten
    }

   if((key==UP)||(key==DOWN))
    {
     for(i=1; i<7; i++)
      {
       result=display[i]; //Kopie der Anzeige
       backup[i]=result; //Kopie der Anzeige
      }

     CheckDisplay();    //Neuen Wert berprfen
     SetPLL();          //PLL einstellen
     if(result>0)       //Bei Fehler
      {
	 //Alte Anzeige wiederherstellen
       for(i=1; i<7; i++)
        {
         result=backup[i]; 
         display[i]=result; 
        }
         
       ShowDisplay();   //Und anzeigen
      }
    }

//Hier STORE Taste auswerten und EEPROM beschreiben
   if(key==STORE)
    {
     LCDPos(1,13);
     LCDWrite("EEP");
     //Letztes Display speichern
     for(i=1; i<7; i++)
      {
       result=display[i]; //WriteEEProm geht nicht mit Arrays !!
       Write_EEProm(i,result);
      }

     Write_EEProm(0,stelle); //Cursorposition merken
     Delay1ms(255);          //Anzeige "EEP" etwas verlngern
     
     LCDPos(1,13);
     LCDWrite("   ");        //"EEP" wieder entfernen
     if(stelle==6) LCDPos(1,7); //Zurck an alte Cursorposition
     else LCDPos(1,stelle);
    }

  } // Ende Programm Endlosschleife
}

